const express = require("express");
const db = require("../config/db");
const multer = require("multer");
const path = require("path");
const authMiddleware = require("../middleware/authMiddleware");

const router = express.Router();

// 📌 ให้ Express ให้บริการไฟล์รูปภาพจากโฟลเดอร์ uploads
router.use("/uploads", express.static(path.join(__dirname, "../uploads")));

// 🔹 Upload Image Config
const storage = multer.diskStorage({
  destination: "./uploads/",
  filename: (req, file, cb) => {
    cb(null, file.fieldname + "-" + Date.now() + path.extname(file.originalname));
  },
});
const upload = multer({ storage });


// 🔹 API โหลดห้องเรียนตามชั้น (ใหม่)
router.get("/rooms/:floor", authMiddleware, (req, res) => {
    const { floor } = req.params;
    db.query("SELECT * FROM rooms WHERE floor = ?", [floor], (err, results) => {
        if (err) return res.status(500).json({ error: err.message });
        res.json(results);
    });
});

// 🔹 Create Report (ใช้ user_id จาก Token)
router.post("/", authMiddleware, upload.single("image"), (req, res) => {
    const { room_id, title, description } = req.body;
    const user_id = req.user.id; // ✅ ดึง user_id จาก Token
    const imageUrl = req.file ? `/uploads/${req.file.filename}` : null;

    db.query(
        "INSERT INTO reports (user_id, room_id, title, description, image_url) VALUES (?, ?, ?, ?, ?)",
        [user_id, room_id, title, description, imageUrl],
        (err, result) => {
            if (err) return res.status(500).json({ error: err.message });
            res.status(201).json({ message: "Report Created!", report_id: result.insertId });
        }
    );
});

// 🔹 Get All Reports (รวมชื่อผู้ใช้, ชื่อห้อง และชั้น พร้อมรองรับการกรองตาม search, floor และ room)
router.get("/", authMiddleware, (req, res) => {
    const search = req.query.search || "";
    const floor = req.query.floor;  // optional: ชั้นที่เลือก
    const room = req.query.room;    // optional: ห้องที่เลือก

    const BASE_URL = "https://repairsys.th2.melon.cloud";  

    let conditions = ["(reports.title LIKE ? OR reports.description LIKE ?)"];
    let params = [`%${search}%`, `%${search}%`];

    if (floor) {
        conditions.push("rooms.floor = ?");
        params.push(floor);
    }
    if (room) {
        conditions.push("reports.room_id = ?");
        params.push(room);
    }

    const query = `
        SELECT reports.*, users.username, rooms.room_name, rooms.floor
        FROM reports
        JOIN users ON reports.user_id = users.id
        JOIN rooms ON reports.room_id = rooms.id
        WHERE ${conditions.join(" AND ")}
    `;

    db.query(query, params, (err, results) => {
        if (err) return res.status(500).json({ error: err.message });

        results.forEach(report => {
            if (report.image_url) {
                report.image_url = `${BASE_URL}${report.image_url}`;
            } else {
                report.image_url = null;  // 🔥 ป้องกันกรณีไม่มีรูปภาพ
            }
        });

        res.json(results);
    });
});





// 🔹 Get Single Report by ID
router.get("/:id", authMiddleware, (req, res) => {
    const BASE_URL = "https://repairsys.th2.melon.cloud";  

    db.query(`
        SELECT reports.*, users.username, rooms.room_name, rooms.floor
        FROM reports
        JOIN users ON reports.user_id = users.id
        JOIN rooms ON reports.room_id = rooms.id
        WHERE reports.id = ?`, 
        [req.params.id], 
        (err, results) => {
          if (err) return res.status(500).json({ error: err.message });
          if (results.length === 0) return res.status(404).json({ error: "Report Not Found" });
  
          const report = results[0];
          if (report.image_url) {
              report.image_url = `${BASE_URL}${report.image_url}`;
          } else {
              report.image_url = null;  // 🔥 ป้องกันกรณีไม่มีรูปภาพ
          }
  
          res.json(report);
    });
});


// 🔹 Update Report (แก้ไขเฉพาะ Title, Description)
router.put("/:id", authMiddleware, (req, res) => {
    const { title, description, room_id } = req.body;
    db.query(
        "UPDATE reports SET title=?, description=?, room_id=? WHERE id=?",
        [title, description, room_id, req.params.id],
        (err, result) => {
            if (err) return res.status(500).json({ error: err.message });
            res.json({ message: "Report Updated!" });
        }
    );
});

// 🔹 Delete Report
router.delete("/:id", authMiddleware, (req, res) => {
    db.query("DELETE FROM reports WHERE id=?", [req.params.id], (err, result) => {
        if (err) return res.status(500).json({ error: err.message });
        res.json({ message: "Report Deleted!" });
    });
});

module.exports = router;
